@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
@@ -71,7 +71,7 @@ inline ssize_t sendmsg(int sockfd, const struct msghdr *msg, int flags)
 #define LONG_CHECK_VALID_INT(l) \
 	do { \
 		if ((l) < INT_MIN && (l) > INT_MAX) { \
-			php_error_docref0(NULL TSRMLS_CC, E_WARNING, "The value %ld does not fit inside " \
+			php_error_docref0(NULL, E_WARNING, "The value " ZEND_LONG_FMT " does not fit inside " \
 					"the boundaries of a native integer", (l)); \
 			return; \
 		} \
@@ -83,6 +83,10 @@ static struct {
 } ancillary_registry;
 
 
+static void ancillary_registery_free_elem(zval *el) {
+	pefree(Z_PTR_P(el), 1);
+}
+
 #ifdef ZTS
 static MUTEX_T ancillary_mutex;
 #endif
@@ -92,7 +96,7 @@ static void init_ancillary_registry(void)
 	anc_reg_key key;
 	ancillary_registry.initialized = 1;
 
-	zend_hash_init(&ancillary_registry.ht, 32, NULL, NULL, 1);
+	zend_hash_init(&ancillary_registry.ht, 32, NULL, ancillary_registery_free_elem, 1);
 
 #define PUT_ENTRY(sizev, var_size, calc, from, to, level, type) \
 	entry.size			= sizev; \
@@ -102,8 +106,7 @@ static void init_ancillary_registry(void)
 	entry.to_array		= to; \
 	key.cmsg_level		= level; \
 	key.cmsg_type		= type; \
-	zend_hash_update(&ancillary_registry.ht, (char*)&key, sizeof(key), \
-			(void*)&entry, sizeof(entry), NULL)
+	zend_hash_str_update_mem(&ancillary_registry.ht, (char*)&key, sizeof(key) - 1, (void*)&entry, sizeof(entry))
 
 #if defined(IPV6_PKTINFO) && HAVE_IPV6
 	PUT_ENTRY(sizeof(struct in6_pktinfo), 0, 0, from_zval_write_in6_pktinfo,
@@ -153,8 +156,7 @@ ancillary_reg_entry *get_ancillary_reg_entry(int cmsg_level, int msg_type)
 	tsrm_mutex_unlock(ancillary_mutex);
 #endif
 
-	if (zend_hash_find(&ancillary_registry.ht, (char*)&key, sizeof(key),
-			(void**)&entry) == SUCCESS) {
+	if ((entry = zend_hash_str_find_ptr(&ancillary_registry.ht, (char*)&key, sizeof(key) - 1)) != NULL) {
 		return entry;
 	} else {
 		return NULL;
@@ -165,7 +167,7 @@ PHP_FUNCTION(socket_sendmsg)
 {
 	zval			*zsocket,
 					*zmsg;
-	long			flags = 0;
+	zend_long			flags = 0;
 	php_socket		*php_sock;
 	struct msghdr	*msghdr;
 	zend_llist		*allocations;
@@ -173,20 +175,22 @@ PHP_FUNCTION(socket_sendmsg)
 	ssize_t			res;
 
 	/* zmsg should be passed by ref */
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ra|l", &zsocket, &zmsg, &flags) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ra|l", &zsocket, &zmsg, &flags) == FAILURE) {
 		return;
 	}
 
 	LONG_CHECK_VALID_INT(flags);
 
-	ZEND_FETCH_RESOURCE(php_sock, php_socket *, &zsocket, -1,
-			php_sockets_le_socket_name, php_sockets_le_socket());
+	if ((php_sock = (php_socket *)zend_fetch_resource(Z_RES_P(zsocket),
+					php_sockets_le_socket_name, php_sockets_le_socket())) == NULL) {
+		RETURN_FALSE;
+	}
 
 	msghdr = from_zval_run_conversions(zmsg, php_sock, from_zval_write_msghdr_send,
 			sizeof(*msghdr), "msghdr", &allocations, &err);
 
 	if (err.has_error) {
-		err_msg_dispose(&err TSRMLS_CC);
+		err_msg_dispose(&err);
 		RETURN_FALSE;
 	}
 
@@ -196,7 +200,7 @@ PHP_FUNCTION(socket_sendmsg)
 		zend_llist_destroy(allocations);
 		efree(allocations);
 
-		RETURN_LONG((long)res);
+		RETURN_LONG((zend_long)res);
 	} else {
 		PHP_SOCKET_ERROR(php_sock, "error in sendmsg", errno);
 		RETURN_FALSE;
@@ -207,7 +211,7 @@ PHP_FUNCTION(socket_recvmsg)
 {
 	zval			*zsocket,
 					*zmsg;
-	long			flags = 0;
+	zend_long			flags = 0;
 	php_socket		*php_sock;
 	ssize_t			res;
 	struct msghdr	*msghdr;
@@ -215,28 +219,30 @@ PHP_FUNCTION(socket_recvmsg)
 	struct err_s	err = {0};
 
 	//ssize_t recvmsg(int sockfd, struct msghdr *msg, int flags);
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ra|l",
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ra/|l",
 			&zsocket, &zmsg, &flags) == FAILURE) {
 		return;
 	}
 
 	LONG_CHECK_VALID_INT(flags);
 
-	ZEND_FETCH_RESOURCE(php_sock, php_socket *, &zsocket, -1,
-			php_sockets_le_socket_name, php_sockets_le_socket());
+	if ((php_sock = (php_socket *)zend_fetch_resource(Z_RES_P(zsocket),
+					php_sockets_le_socket_name, php_sockets_le_socket())) == NULL) {
+		RETURN_FALSE;
+	}
 
 	msghdr = from_zval_run_conversions(zmsg, php_sock, from_zval_write_msghdr_recv,
 			sizeof(*msghdr), "msghdr", &allocations, &err);
 
 	if (err.has_error) {
-		err_msg_dispose(&err TSRMLS_CC);
+		err_msg_dispose(&err);
 		RETURN_FALSE;
 	}
 
 	res = recvmsg(php_sock->bsd_socket, msghdr, (int)flags);
 
 	if (res != -1) {
-		zval *zres;
+		zval *zres, tmp;
 		struct key_value kv[] = {
 				{KEY_RECVMSG_RET, sizeof(KEY_RECVMSG_RET), &res},
 				{0}
@@ -244,7 +250,7 @@ PHP_FUNCTION(socket_recvmsg)
 
 
 		zres = to_zval_run_conversions((char *)msghdr, to_zval_read_msghdr,
-				"msghdr", kv, &err);
+				"msghdr", kv, &err, &tmp);
 
 		/* we don;t need msghdr anymore; free it */
 		msghdr = NULL;
@@ -253,31 +259,30 @@ PHP_FUNCTION(socket_recvmsg)
 		zval_dtor(zmsg);
 		if (!err.has_error) {
 			ZVAL_COPY_VALUE(zmsg, zres);
-			efree(zres); /* only shallow destruction */
 		} else {
-			err_msg_dispose(&err TSRMLS_CC);
+			err_msg_dispose(&err);
 			ZVAL_FALSE(zmsg);
 			/* no need to destroy/free zres -- it's NULL in this circumstance */
 			assert(zres == NULL);
 		}
 	} else {
 		SOCKETS_G(last_error) = errno;
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "error in recvmsg [%d]: %s",
-				errno, sockets_strerror(errno TSRMLS_CC));
+		php_error_docref(NULL, E_WARNING, "error in recvmsg [%d]: %s",
+				errno, sockets_strerror(errno));
 		RETURN_FALSE;
 	}
 
-	RETURN_LONG((long)res);
+	RETURN_LONG((zend_long)res);
 }
 
 PHP_FUNCTION(socket_cmsg_space)
 {
-	long				level,
+	zend_long				level,
 						type,
 						n = 0;
 	ancillary_reg_entry	*entry;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ll|l",
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ll|l",
 			&level, &type, &n) == FAILURE) {
 		return;
 	}
@@ -287,31 +292,31 @@ PHP_FUNCTION(socket_cmsg_space)
 	LONG_CHECK_VALID_INT(n);
 
 	if (n < 0) {
-		php_error_docref0(NULL TSRMLS_CC, E_WARNING, "The third argument "
+		php_error_docref0(NULL, E_WARNING, "The third argument "
 				"cannot be negative");
 		return;
 	}
 
 	entry = get_ancillary_reg_entry(level, type);
 	if (entry == NULL) {
-		php_error_docref0(NULL TSRMLS_CC, E_WARNING, "The pair level %ld/type %ld is "
+		php_error_docref0(NULL, E_WARNING, "The pair level " ZEND_LONG_FMT "/type " ZEND_LONG_FMT " is "
 				"not supported by PHP", level, type);
 		return;
 	}
 
-	if (entry->var_el_size > 0 && n > (LONG_MAX - (long)entry->size -
-			(long)CMSG_SPACE(0) - 15L) / entry->var_el_size) {
+	if (entry->var_el_size > 0 && n > (zend_long)((ZEND_LONG_MAX - entry->size -
+			CMSG_SPACE(0) - 15L) / entry->var_el_size)) {
 		/* the -15 is to account for any padding CMSG_SPACE may add after the data */
-		php_error_docref0(NULL TSRMLS_CC, E_WARNING, "The value for the "
-				"third argument (%ld) is too large", n);
+		php_error_docref0(NULL, E_WARNING, "The value for the "
+				"third argument (" ZEND_LONG_FMT ") is too large", n);
 		return;
 	}
 
-	RETURN_LONG((long)CMSG_SPACE(entry->size + n * entry->var_el_size));
+	RETURN_LONG((zend_long)CMSG_SPACE(entry->size + n * entry->var_el_size));
 }
 
 #if HAVE_IPV6
-int php_do_setsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname, zval **arg4 TSRMLS_DC)
+int php_do_setsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname, zval *arg4)
 {
 	struct err_s	err = {0};
 	zend_llist		*allocations = NULL;
@@ -325,8 +330,8 @@ int php_do_setsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname,
 #ifdef IPV6_PKTINFO
 	case IPV6_PKTINFO:
 #ifdef PHP_WIN32
-		if (Z_TYPE_PP(arg4) == IS_ARRAY) {
-			php_error_docref0(NULL TSRMLS_CC, E_WARNING, "Windows does not "
+		if (Z_TYPE_P(arg4) == IS_ARRAY) {
+			php_error_docref0(NULL, E_WARNING, "Windows does not "
 					"support sticky IPV6_PKTINFO");
 			return FAILURE;
 		} else {
@@ -336,10 +341,10 @@ int php_do_setsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname,
 			return 1;
 		}
 #endif
-		opt_ptr = from_zval_run_conversions(*arg4, php_sock, from_zval_write_in6_pktinfo,
+		opt_ptr = from_zval_run_conversions(arg4, php_sock, from_zval_write_in6_pktinfo,
 				sizeof(struct in6_pktinfo),	"in6_pktinfo", &allocations, &err);
 		if (err.has_error) {
-			err_msg_dispose(&err TSRMLS_CC);
+			err_msg_dispose(&err);
 			return FAILURE;
 		}
 
@@ -362,7 +367,7 @@ dosockopt:
 	return retval != 0 ? FAILURE : SUCCESS;
 }
 
-int php_do_getsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname, zval *result TSRMLS_DC)
+int php_do_getsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname, zval *result)
 {
 	struct err_s		err = {0};
 	void				*buffer;
@@ -388,14 +393,14 @@ int php_do_getsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname,
 	if (res != 0) {
 		PHP_SOCKET_ERROR(php_sock, "unable to get socket option", errno);
 	} else {
+		zval tmp;
 		zval *zv = to_zval_run_conversions(buffer, reader, "in6_pktinfo",
-				empty_key_value_list, &err);
+				empty_key_value_list, &err, &tmp);
 		if (err.has_error) {
-			err_msg_dispose(&err TSRMLS_CC);
+			err_msg_dispose(&err);
 			res = -1;
 		} else {
 			ZVAL_COPY_VALUE(result, zv);
-			efree(zv);
 		}
 	}
 	efree(buffer);
