@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2017 The PHP Group                                |
+   | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -533,10 +533,10 @@ php_stream * php_stream_url_wrap_ftp(php_stream_wrapper *wrapper, const char *pa
 			(tmpzval = php_stream_context_get_option(context, "ftp", "resume_pos")) != NULL &&
 			Z_TYPE_P(tmpzval) == IS_LONG &&
 			Z_LVAL_P(tmpzval) > 0) {
-			php_stream_printf(stream, "REST %pd\r\n", Z_LVAL_P(tmpzval));
+			php_stream_printf(stream, "REST " ZEND_LONG_FMT "\r\n", Z_LVAL_P(tmpzval));
 			result = GET_FTP_RESULT(stream);
 			if (result < 300 || result > 399) {
-				php_stream_wrapper_log_error(wrapper, options, "Unable to resume from offset %pd", Z_LVAL_P(tmpzval));
+				php_stream_wrapper_log_error(wrapper, options, "Unable to resume from offset " ZEND_LONG_FMT, Z_LVAL_P(tmpzval));
 				goto errexit;
 			}
 		}
@@ -833,11 +833,11 @@ static int php_stream_ftp_url_stat(php_stream_wrapper *wrapper, const char *url,
 		struct tm tm, tmbuf, *gmt;
 		time_t stamp;
 
-		while (p - tmp_line < sizeof(tmp_line) && !isdigit(*p)) {
+		while ((size_t)(p - tmp_line) < sizeof(tmp_line) && !isdigit(*p)) {
 			p++;
 		}
 
-		if (p - tmp_line > sizeof(tmp_line)) {
+		if ((size_t)(p - tmp_line) > sizeof(tmp_line)) {
 			goto mdtm_error;
 		}
 
@@ -1182,7 +1182,8 @@ static php_stream_wrapper_ops ftp_stream_wops = {
 	php_stream_ftp_unlink, /* unlink */
 	php_stream_ftp_rename, /* rename */
 	php_stream_ftp_mkdir,  /* mkdir */
-	php_stream_ftp_rmdir   /* rmdir */
+	php_stream_ftp_rmdir,  /* rmdir */
+	NULL
 };
 
 PHPAPI php_stream_wrapper php_stream_ftp_wrapper =	{
