@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2017 The PHP Group                                |
+   | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -33,7 +33,7 @@ extern zend_module_entry sysvshm_module_entry;
 
 #ifdef PHP_WIN32
 # include <TSRM/tsrm_win32.h>
-typedef int key_t;
+# include "win32/ipc.h"
 # ifndef THREAD_LS
 #  define THREAD_LS
 # endif
